"""
    File for setting the view classes used from the api django app.
"""

from datetime import datetime, timezone

from shared_models.models import ButtonBox, ButtonBoxUse
from shared_models.serializers import ButtonBoxSerializer, ButtonBoxUseSerializer

from rest_framework import status
from rest_framework.response import Response
from rest_framework.views import APIView
from rest_framework.renderers import JSONRenderer
from rest_framework_simplejwt.authentication import JWTAuthentication


###
### HELPERS
###
def helper_get_from_parsed(parsed_name: str, parsed_value: str, parsed_type: str):
    """Helper method used from the ButtonBoxUseView class for setting the return values from different 'parsed_name' and 'parsed_type'.
    It will fetch or update the values from the ButtonBoxUse model.

    Parameters
    ----------
    parsed_name: str
        String with the name parsed.
    parsed_value: str
        String with the value parsed.
    parsed_type: str
        String with the type parsed.

    Returns
    -------
    ret_dict: dict
        The Dictionary with the output from executing this helper method.
    ret_status: rest_framework.status
        The HTTP status from executing this helper method.
    """

    founded_box = None
    ret_dict = None
    ret_status = status.HTTP_500_INTERNAL_SERVER_ERROR
    try:
        if parsed_name == "id":
            founded_box = ButtonBox.objects.get(id=parsed_value)

        elif parsed_name == "name":
            founded_box = ButtonBox.objects.get(name=parsed_value)

        elif parsed_name == "pin_button":
            founded_box = ButtonBox.objects.get(pin_button=parsed_value)

        elif parsed_name == "pin_led":
            founded_box = ButtonBox.objects.get(pin_led=parsed_value)

        else:
            ret_dict = {
                'error': True,
                'error_msg': f"ERROR: unknown parameter parsed: {parsed_name}."
            }
            ret_status = status.HTTP_406_NOT_ACCEPTABLE

    except ButtonBox.DoesNotExist:
        ret_dict = {
            'error': True,
            'error_msg': f"No Box could be founded with the '{parsed_name}': {parsed_value}"
        }
        ret_status = status.HTTP_404_NOT_FOUND

    except Exception as e:
        ret_dict = {
            'error': True,
            'error_msg': f"ERROR happend when trying to search for the box with {parsed_name}: {parsed_value} | Error: {e}"
        }
        ret_status = status.HTTP_406_NOT_ACCEPTABLE

    ### If error ocurred, finish
    if ret_dict is not None:
        pass

    else:
        if parsed_type == "add":
            try:
                ### Add pushed from pin
                added = ButtonBoxUse(box_used=founded_box)
                added.save()

                ret_dict = {
                    'error': False,
                    'message': "Added button press",
                    "button-used": {
                        "id": added.id,
                        "box_id": added.box_used.id,
                        'box_name': added.box_used.name,
                        'box_pin_button': added.box_used.pin_button,
                        'box_pin_led': added.box_used.pin_led,
                        'date': str(added.date)
                    }
                }
                ret_status = status.HTTP_200_OK


            except Exception as e:
                ret_dict = {
                    'error': True,
                    'message': f"Error saving the press button with output: {e}",
                }
                ret_status = status.HTTP_418_IM_A_TEAPOT

        elif parsed_type == "list":
            try:
                ### Get list pushed button
                box_uses = ButtonBoxUse.objects.filter(box_used=founded_box)
                serializer = ButtonBoxUseSerializer(box_uses, many=True)
                ret_dict = serializer.data
                ret_status = status.HTTP_200_OK

            except Exception as e:
                ret_dict = {
                    'error': True,
                    'error_msg': f"ERROR happend when trying to search for the box with {parsed_name}: {parsed_value} | Error: {e}"
                }
                ret_status = status.HTTP_406_NOT_ACCEPTABLE

        else:
            ret_dict = {
                'error': True,
                'error_msg': "ERROR parsing response from 'helper_get_from_parsed' function"
            }
            ret_status = status.HTTP_501_NOT_IMPLEMENTED

    return ret_dict, ret_status


###
### VIEW CLASSES
###
class ButtonBoxSearchView(APIView):
    """Class for setting the API views for Searching at the ButtonBox model.
    """

    # permission_classes = [IsAuthenticated]
    authentication_classes = [JWTAuthentication]
    renderer_classes = [JSONRenderer]

    def get(self, request, format=None):
        """Method for dealing with the GET requests done at this view.

        Parameters
        ----------
        request: rest_framework.request.Request
            The parameters from the GET request.
        format: None
            If some format was sent.

        Returns
        -------
        rest_framework.response.Response
            The response object.
        """
        # Get parsed
        box_id = request.query_params.get("id")
        box_name = request.query_params.get("name")
        box_pin_button = request.query_params.get("pin_button")
        box_pin_led = request.query_params.get("pin_led")

        if box_id is not None and box_name is None and box_pin_button is None and box_pin_led is None:
            box_founded = ButtonBox.objects.filter(id__icontains=box_id)

        elif box_id is None and box_name is not None and box_pin_button is None and box_pin_led is None:
            box_founded = ButtonBox.objects.filter(name__icontains=box_name)

        elif box_id is None and box_name is None and box_pin_button is not None and box_pin_led is None:
            box_founded = ButtonBox.objects.filter(pin_button__icontains=box_pin_button)

        elif box_id is None and box_name is None and box_pin_button is None and box_pin_led is not None:
            box_founded = ButtonBox.objects.filter(pin_led__icontains=box_pin_led)

        # If not, return all
        else:
            box_founded = ButtonBox.objects.all()

        serializer = ButtonBoxSerializer(box_founded, many=True)

        return Response(serializer.data, status=status.HTTP_200_OK)


class ButtonBoxUseView(APIView):
    """Class for setting the API views for Searching and Adding values at the ButtonBoxUse model.
    """

    # permission_classes = [IsAuthenticated]
    authentication_classes = [JWTAuthentication]
    renderer_classes = [JSONRenderer]

    def get(self, request, format=None):
        """Method for dealing with the GET requests done at this view.

        Parameters
        ----------
        request: rest_framework.request.Request
            The parameters from the GET request.
        format: None
            If some format was sent.

        Returns
        -------
        rest_framework.response.Response
            The response object.
        """

        # Get parsed
        box_id = request.query_params.get("id")
        box_name = request.query_params.get("name")
        box_pin_button = request.query_params.get("pin_button")
        box_pin_led = request.query_params.get("pin_led")

        if box_id is not None and box_name is None and box_pin_button is None and box_pin_led is None:
            founded_box = ButtonBox.objects.get(id=box_id)

        elif box_id is None and box_name is not None and box_pin_button is None and box_pin_led is None:
            founded_box = ButtonBox.objects.get(name=box_name)

        elif box_id is None and box_name is None and box_pin_button is not None and box_pin_led is None:
            founded_box = ButtonBox.objects.get(pin_button=box_pin_button)

        elif box_id is None and box_name is None and box_pin_button is None and box_pin_led is not None:
            founded_box = ButtonBox.objects.get(pin_led=box_pin_led)

        else:
            founded_box = None

        # Get from parsed, if None get all
        if founded_box is None:
            boxes_uses = ButtonBoxUse.objects.all()
            serializer = ButtonBoxUseSerializer(boxes_uses, many=True)

        else:
            box_uses = ButtonBoxUse.objects.filter(box_used=founded_box)
            serializer = ButtonBoxUseSerializer(box_uses, many=True)

        return Response(serializer.data, status=status.HTTP_200_OK)

    def post(self, request, format=None):
        """Method for dealing with the POST requests done at this view.

        Parameters
        ----------
        request: rest_framework.request.Request
            The parameters from the POST request.
        format: None
            If some format was sent.

        Returns
        -------
        rest_framework.response.Response
            The response object.
        """
        # Get the box name from the query parameters (if Nonne, default to empty str)
        parsed_id = request.data.get("id")
        parsed_name = request.data.get("name")
        # parsed_pin_number = request.data.get("pin_number")
        parsed_pin_button = request.data.get("pin_button")
        parsed_pin_led = request.data.get("pin_led")

        if parsed_id is None and parsed_name is None and parsed_pin_button is None and parsed_pin_led is None:
            ret_dict = {
                "error": True,
                "message": "Unable to add box used from this end-point withtout parsed info from 'name', 'pin_button' or 'pin_led'"
            }
            ret_status = status.HTTP_400_BAD_REQUEST

        elif parsed_id is not None and parsed_name is not None and parsed_pin_button is not None and parsed_pin_led is not None:
            ret_dict = {
                "error": True,
                "message": "Unable to add box used from this end-point with MULTIPLE parsed infos from 'id', 'name', 'pin_button' and 'pin_led'. Use only one"
            }
            ret_status = status.HTTP_400_BAD_REQUEST

        ### ID
        elif parsed_id is not None and parsed_name is None and parsed_pin_button is None and parsed_pin_led is None:
            ret_dict, ret_status = helper_get_from_parsed(
                parsed_name='id',
                parsed_value=parsed_id,
                parsed_type='add'
            )

        ### NAME
        elif parsed_name is not None and parsed_pin_button is None and parsed_pin_led is None:
            ret_dict, ret_status = helper_get_from_parsed(
                parsed_name='name',
                parsed_value=parsed_name,
                parsed_type='add'
            )

        ### PIN_BUTTON
        elif parsed_name is None and parsed_pin_button is not None and parsed_pin_led is None:
            ret_dict, ret_status = helper_get_from_parsed(
                parsed_name='pin_button',
                parsed_value=parsed_pin_button,
                parsed_type='add'
            )

        ### PIN_LED
        elif parsed_name is None and parsed_pin_button is None and parsed_pin_led is not None:
            ret_dict, ret_status = helper_get_from_parsed(
                parsed_name='pin_led',
                parsed_value=parsed_pin_led,
                parsed_type='add'
            )

        else:
            ret_dict = {
                "error": True,
                "message": "Option not implemented..."
            }
            ret_status = status.HTTP_501_NOT_IMPLEMENTED

        return Response(ret_dict, status=ret_status)


class ButtonBoxUseCountView(APIView):
    """Class for setting the API views for getting the amount of uses from a Button Box.
    It is mostly used from Grafana for getting the fancy requests from date windows.
    """

    # permission_classes = [IsAuthenticated]
    authentication_classes = [JWTAuthentication]
    renderer_classes = [JSONRenderer]

    def get(self, request, format=None):
        """Method for dealing with the GET requests done at this view.

        Parameters
        ----------
        request: rest_framework.request.Request
            The parameters from the GET request.
        format: None
            If some format was sent.

        Returns
        -------
        rest_framework.response.Response
            The response object.
        """

        ###
        ### EXAMPLE of the grafana used URL ;)
        ###
        ### http://localhost:8000/api/box/use/count?from=${__from}&to=${__to}
        ###

        ### GET PARSED
        query_param_from = request.query_params.get("from")
        query_param_to = request.query_params.get("to")

        if query_param_from is not None and query_param_to is None:
            ret_list = [
                {
                    'error': True,
                    'error_msg': "Query parameter for 'from' is parsed and 'to' is not parsed. You have to use BOUTH or NONE"
                }
            ]
            ret_status = status.HTTP_400_BAD_REQUEST

        elif query_param_from is None and query_param_to is not None:
            ret_list = [
                {
                    'error': True,
                    'error_msg': "Query parameter for 'to' is parsed and 'from' is not parsed. You have to use BOUTH or NONE"
                }
            ]
            ret_status = status.HTTP_400_BAD_REQUEST

        ### RETURN ALL
        elif query_param_from is None and query_param_to is None:

            ret_list = []
            for button_box in ButtonBox.objects.all():
                ret_list.append(
                    {
                        "name": button_box.name,
                        "count": ButtonBoxUse.objects.filter(box_used=button_box).count()
                    }
                )

            ret_status = status.HTTP_200_OK

        ### RETURN FROM TIME FRAME
        elif query_param_from is not None and query_param_to is not None:

            try:
                date_from = datetime.fromtimestamp(int(query_param_from) / 1000, timezone.utc)

            except Exception as e:
                ret_list = [
                    {
                        'error': True,
                        'error_msg': f"Error parsing epoch time to datetime from parsed 'date_from' parameter with error: {e}"
                    }
                ]
                ret_status = status.HTTP_400_BAD_REQUEST

            try:
                date_to = datetime.fromtimestamp(int(query_param_to) / 1000, timezone.utc)

            except Exception as e:
                ret_list = [
                    {
                        'error': True,
                        'error_msg': f"Error parsing epoch time to datetime from parsed 'date_to' parameter with error: {e}"
                    }
                ]
                ret_status = status.HTTP_400_BAD_REQUEST

            ret_list = []
            for button_box in ButtonBox.objects.all():
                button_name = button_box.name
                count = 0
                for button_use in ButtonBoxUse.objects.filter(date__gt=date_from, date__lt=date_to):
                    if button_name == button_use.box_name:
                        count += 1
                        ret_list.append(
                            {
                                "name": button_name,
                                "count": count,
                                "date": button_use.date
                            }
                        )
            ret_status = status.HTTP_200_OK

        else:
            ret_list = [
                {
                    'error': True,
                    'error_msg': "Not implemented"
                }
            ]
            ret_status = status.HTTP_501_NOT_IMPLEMENTED

        return Response(ret_list, status=ret_status)
